'use strict';

Object.defineProperty(exports, "__esModule", {
    value: true
});

var _fs = require('fs');

var _fs2 = _interopRequireDefault(_fs);

var _path = require('path');

var _path2 = _interopRequireDefault(_path);

var _babelTypes = require('babel-types');

var t = _interopRequireWildcard(_babelTypes);

var _babylon = require('babylon');

var babylon = _interopRequireWildcard(_babylon);

function _interopRequireWildcard(obj) { if (obj && obj.__esModule) { return obj; } else { var newObj = {}; if (obj != null) { for (var key in obj) { if (Object.prototype.hasOwnProperty.call(obj, key)) newObj[key] = obj[key]; } } newObj.default = obj; return newObj; } }

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

var typeInfoRegex = /^:([a-z])(\((.+)\))?/;
var paramRegex = /\$\{(\d+)\}/g;

var buildKey = function buildKey(literals) {
    var stripType = function stripType(s) {
        return s.replace(typeInfoRegex, '');
    };
    var lastPartialKey = stripType(literals[literals.length - 1]);
    var prependPartialKey = function prependPartialKey(memo, curr, i) {
        return stripType(curr) + '${' + i + '}' + memo;
    };
    return literals.slice(0, -1).reduceRight(prependPartialKey, lastPartialKey);
};

var updateTemplateLiterals = {
    TaggedTemplateExpression: function TaggedTemplateExpression(path, _ref) {
        var translations = _ref.translations;

        if (!path.node.translated && path.node.quasi && path.node.tag.name === 'i18n' && path.node.quasi.quasis) {
            var quasis = path.node.quasi.quasis.map(function (n) {
                return n.value.raw;
            });
            var ext = quasis.map(function (q) {
                var match = q.match(typeInfoRegex);
                if (match && match.length > 1) {
                    return match[0];
                }
                return '';
            });
            var key = buildKey(quasis);
            var translation = translations[key];
            if (translation) {
                var indices = [];
                var paramMatch = void 0;
                while ((paramMatch = paramRegex.exec(translation)) && paramMatch.length == 2) {
                    indices.push(Number.parseInt(paramMatch[1]));
                }
                var literals = translation.split(/\$\{\d+\}/g);
                var _quasis = [];
                var expressions = [];
                var nextPrefix = '';
                for (var i = 0; i < literals.length; i++) {
                    _quasis.push(t.templateElement({ raw: nextPrefix + literals[i] }, i === literals.length - 1));
                    if (indices.length > i) {
                        var paramIndex = indices[i];
                        if (ext.length > paramIndex + 1) {
                            nextPrefix = ext[paramIndex + 1];
                        } else {
                            nextPrefix = '';
                        }
                        if (path.node.quasi.expressions.length > paramIndex) {
                            var expression = path.node.quasi.expressions[paramIndex];
                            expressions.push(expression);
                        }
                    }
                }
                var literal = t.templateLiteral(_quasis, expressions);
                var taggedTemplate = t.taggedTemplateExpression(path.node.tag, literal);
                taggedTemplate.translated = true;
                path.replaceWith(taggedTemplate);
            }
        }
    }
};

var bla = function bla() {
    return {
        visitor: {
            Program: function Program(p, _ref2) {
                var opts = _ref2.opts;

                if (opts['config']) {
                    // TODO Clean up this mess
                    var code = 'let configs = ' + JSON.stringify(opts['config']) + ';';
                    var objectExpression = babylon.parse(code).program.body[0].declarations[0].init;
                    var ast = t.expressionStatement(t.callExpression(t.identifier("i18nConfig"), [objectExpression]));
                    p.unshiftContainer('body', ast);
                }

                if (opts['globalImport'] || opts['config']) {
                    var newImport = t.importDeclaration([t.importDefaultSpecifier(t.identifier("i18n")), t.importSpecifier(t.identifier("i18nConfig"), t.identifier("i18nConfig"))], t.stringLiteral("es2015-i18n-tag"));

                    p.unshiftContainer('body', newImport);
                }

                var translations = {};
                if (opts.translation) {
                    try {
                        var translationsFile = _path2.default.resolve(__dirname, opts.translation);
                        console.log('Reading: ' + translationsFile + ' ...');
                        translations = JSON.parse(_fs2.default.readFileSync(translationsFile, 'utf-8'));
                    } catch (err) {
                        console.warn(err.message);
                    }
                }
                p.traverse(updateTemplateLiterals, { translations: translations });
            }
        }
    };
};

exports.default = bla;